//-----------------------------------------------------
// author: "Albert Graef"
// name: "chimes -- chimes synth using a bank of resonz filters"
// version: "2.0"
//
// Code generated with Faust 2.0.a30 (http://faust.grame.fr)
//-----------------------------------------------------

#ifndef  __mydsp_H__
#define  __mydsp_H__
/************************************************************************
 ************************************************************************
    FAUST Architecture File
	Copyright (C) 2006-2011 Albert Graef <Dr.Graef@t-online.de>
    ---------------------------------------------------------------------
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
	published by the Free Software Foundation; either version 2.1 of the 
	License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
 	License along with the GNU C Library; if not, write to the Free
  	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  	02111-1307 USA. 
 ************************************************************************
 ************************************************************************/

/* Pd architecture file, written by Albert Graef <Dr.Graef@t-online.de>.
   This was derived from minimal.cpp included in the Faust distribution.
   Please note that this is to be compiled as a shared library, which is
   then loaded dynamically by Pd as an external. */

#include <stdlib.h>
#include <math.h>
#include <string>

#include "faust/misc.h"
#include "faust/gui/UI.h"
#include "faust/gui/meta.h"
#include "faust/audio/dsp.h"

//using namespace std;

/******************************************************************************
*******************************************************************************

							       VECTOR INTRINSICS

*******************************************************************************
*******************************************************************************/


/***************************************************************************
   Pd UI interface
 ***************************************************************************/

enum ui_elem_type_t {
  UI_BUTTON, UI_CHECK_BUTTON,
  UI_V_SLIDER, UI_H_SLIDER, UI_NUM_ENTRY,
  UI_V_BARGRAPH, UI_H_BARGRAPH,
  UI_END_GROUP, UI_V_GROUP, UI_H_GROUP, UI_T_GROUP
};

struct ui_elem_t {
  ui_elem_type_t type;
  char *label;
  float *zone;
  float init, min, max, step;
};

class PdUI : public UI
{
public:
  const char *name;
  int nelems, level;
  ui_elem_t *elems;
		
  PdUI();
  PdUI(const char *nm, const char *s);
  virtual ~PdUI();

protected:
  std::string path;
  void add_elem(ui_elem_type_t type, const char *label = NULL);
  void add_elem(ui_elem_type_t type, const char *label, float *zone);
  void add_elem(ui_elem_type_t type, const char *label, float *zone,
		float init, float min, float max, float step);
  void add_elem(ui_elem_type_t type, const char *label, float *zone,
		float min, float max);

public:
  virtual void addButton(const char* label, float* zone);
  virtual void addCheckButton(const char* label, float* zone);
  virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step);
  virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step);
  virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step);

  virtual void addHorizontalBargraph(const char* label, float* zone, float min, float max);
  virtual void addVerticalBargraph(const char* label, float* zone, float min, float max);
  
  virtual void openTabBox(const char* label);
  virtual void openHorizontalBox(const char* label);
  virtual void openVerticalBox(const char* label);
  virtual void closeBox();
	
  virtual void run();
};

static std::string mangle(const char *name, int level, const char *s)
{
  const char *s0 = s;
  std::string t = "";
  if (!s) return t;
  // Get rid of bogus "0x00" labels in recent Faust revisions. Also, for
  // backward compatibility with old Faust versions, make sure that default
  // toplevel groups and explicit toplevel groups with an empty label are
  // treated alike (these both return "0x00" labels in the latest Faust, but
  // would be treated inconsistently in earlier versions).
  if (!*s || strcmp(s, "0x00") == 0) {
    if (level == 0)
      // toplevel group with empty label, map to dsp name
      s = name;
    else
      // empty label
      s = "";
  }
  while (*s)
    if (isalnum(*s))
      t += *(s++);
    else {
      const char *s1 = s;
      while (*s && !isalnum(*s)) ++s;
      if (s1 != s0 && *s) t += "-";
    }
  return t;
}

static std::string normpath(std::string path)
{
  path = std::string("/")+path;
  int pos = path.find("//");
  while (pos >= 0) {
    path.erase(pos, 1);
    pos = path.find("//");
  }
  size_t len = path.length();
  if (len > 1 && path[len-1] == '/')
    path.erase(len-1, 1);
  return path;
}

static std::string pathcat(std::string path, std::string label)
{
  if (path.empty())
    return normpath(label);
  else if (label.empty())
    return normpath(path);
  else
    return normpath(path+"/"+label);
}

PdUI::PdUI()
{
  nelems = level = 0;
  elems = NULL;
  name = "";
  path = "";
}

PdUI::PdUI(const char *nm, const char *s)
{
  nelems = level = 0;
  elems = NULL;
  name = nm?nm:"";
  path = s?s:"";
}

PdUI::~PdUI()
{
  if (elems) {
    for (int i = 0; i < nelems; i++)
      if (elems[i].label)
	free(elems[i].label);
    free(elems);
  }
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = NULL;
  elems[nelems].init = 0.0;
  elems[nelems].min = 0.0;
  elems[nelems].max = 0.0;
  elems[nelems].step = 0.0;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = 0.0;
  elems[nelems].min = 0.0;
  elems[nelems].max = 1.0;
  elems[nelems].step = 1.0;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
			  float init, float min, float max, float step)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = init;
  elems[nelems].min = min;
  elems[nelems].max = max;
  elems[nelems].step = step;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
			  float min, float max)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = 0.0;
  elems[nelems].min = min;
  elems[nelems].max = max;
  elems[nelems].step = 0.0;
  nelems++;
}

void PdUI::addButton(const char* label, float* zone)
{ add_elem(UI_BUTTON, label, zone); }
void PdUI::addCheckButton(const char* label, float* zone)
{ add_elem(UI_CHECK_BUTTON, label, zone); }
void PdUI::addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_V_SLIDER, label, zone, init, min, max, step); }
void PdUI::addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_H_SLIDER, label, zone, init, min, max, step); }
void PdUI::addNumEntry(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_NUM_ENTRY, label, zone, init, min, max, step); }

void PdUI::addHorizontalBargraph(const char* label, float* zone, float min, float max)
{ add_elem(UI_H_BARGRAPH, label, zone, min, max); }
void PdUI::addVerticalBargraph(const char* label, float* zone, float min, float max)
{ add_elem(UI_V_BARGRAPH, label, zone, min, max); }

void PdUI::openTabBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::openHorizontalBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::openVerticalBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::closeBox()
{
  int pos = path.rfind("/");
  if (pos < 0) pos = 0;
  path.erase(pos);
  level--;
}

void PdUI::run() {}

/******************************************************************************
*******************************************************************************

			    FAUST DSP

*******************************************************************************
*******************************************************************************/

//----------------------------------------------------------------------------
//  FAUST generated signal processor
//----------------------------------------------------------------------------
		
#ifndef FAUSTFLOAT
#define FAUSTFLOAT float
#endif  

#include <math.h>

inline float faustpower2_f(float value) {
	return (value * value);
	
}

#ifndef FAUSTCLASS 
#define FAUSTCLASS mydsp
#endif

class mydsp : public dsp {
	
  private:
	
	float fRec0[3];
	float fRec6[3];
	float fRec9[3];
	float fRec12[3];
	float fRec15[3];
	int iRec1[2];
	float fRec2[2];
	int iRec3[2];
	float fVec0[2];
	int iRec4[2];
	float fRec5[2];
	int iRec7[2];
	float fRec8[2];
	int iRec10[2];
	float fRec11[2];
	int iRec13[2];
	float fRec14[2];
	int iRec16[2];
	float fRec17[2];
	int fSamplingFreq;
	int iConst0;
	float fConst1;
	FAUSTFLOAT fHslider0;
	FAUSTFLOAT fHslider1;
	FAUSTFLOAT fEntry0;
	FAUSTFLOAT fEntry1;
	FAUSTFLOAT fEntry2;
	float fConst2;
	float fConst3;
	FAUSTFLOAT fEntry3;
	FAUSTFLOAT fEntry4;
	FAUSTFLOAT fButton0;
	FAUSTFLOAT fEntry5;
	FAUSTFLOAT fEntry6;
	FAUSTFLOAT fEntry7;
	FAUSTFLOAT fEntry8;
	FAUSTFLOAT fEntry9;
	FAUSTFLOAT fEntry10;
	FAUSTFLOAT fEntry11;
	FAUSTFLOAT fEntry12;
	FAUSTFLOAT fEntry13;
	FAUSTFLOAT fEntry14;
	FAUSTFLOAT fEntry15;
	FAUSTFLOAT fEntry16;
	FAUSTFLOAT fEntry17;
	FAUSTFLOAT fEntry18;
	FAUSTFLOAT fEntry19;
	FAUSTFLOAT fEntry20;
	FAUSTFLOAT fEntry21;
	FAUSTFLOAT fEntry22;
	
  public:
	
	void static metadata(Meta* m) { 
		m->declare("author", "Albert Graef");
		m->declare("math.lib/author", "GRAME");
		m->declare("math.lib/copyright", "GRAME");
		m->declare("math.lib/license", "LGPL with exception");
		m->declare("math.lib/name", "Math Library");
		m->declare("math.lib/version", "1.0");
		m->declare("music.lib/author", "GRAME");
		m->declare("music.lib/copyright", "GRAME");
		m->declare("music.lib/license", "LGPL with exception");
		m->declare("music.lib/name", "Music Library");
		m->declare("music.lib/version", "1.0");
		m->declare("name", "chimes -- chimes synth using a bank of resonz filters");
		m->declare("version", "2.0");
	}

	virtual int getNumInputs() {
		return 0;
		
	}
	virtual int getNumOutputs() {
		return 2;
		
	}
	virtual int getInputRate(int channel) {
		int rate;
		switch (channel) {
			default: {
				rate = -1;
				break;
			}
			
		}
		return rate;
		
	}
	virtual int getOutputRate(int channel) {
		int rate;
		switch (channel) {
			case 0: {
				rate = 1;
				break;
			}
			case 1: {
				rate = 1;
				break;
			}
			default: {
				rate = -1;
				break;
			}
			
		}
		return rate;
		
	}
	
	static void classInit(int samplingFreq) {
		
	}
	
	virtual void instanceInit(int samplingFreq) {
		fSamplingFreq = samplingFreq;
		iConst0 = min(192000, max(1, fSamplingFreq));
		fConst1 = float((0.5f * float(iConst0)));
		fHslider0 = FAUSTFLOAT(0.5);
		fHslider1 = FAUSTFLOAT(0.5);
		fEntry0 = FAUSTFLOAT(440.);
		fEntry1 = FAUSTFLOAT(0.167);
		fEntry2 = FAUSTFLOAT(0.002);
		fConst2 = float((3.14159f / float(iConst0)));
		fConst3 = float((6.28319f / float(iConst0)));
		fEntry3 = FAUSTFLOAT(1.);
		fEntry4 = FAUSTFLOAT(1.);
		fButton0 = FAUSTFLOAT(0.);
		for (int i0 = 0; (i0 < 2); i0 = (i0 + 1)) {
			iRec1[i0] = 0;
			
		}
		fEntry5 = FAUSTFLOAT(0.01);
		for (int i1 = 0; (i1 < 2); i1 = (i1 + 1)) {
			fRec2[i1] = 0.f;
			
		}
		for (int i2 = 0; (i2 < 2); i2 = (i2 + 1)) {
			iRec3[i2] = 0;
			
		}
		for (int i3 = 0; (i3 < 2); i3 = (i3 + 1)) {
			fVec0[i3] = 0.f;
			
		}
		for (int i4 = 0; (i4 < 3); i4 = (i4 + 1)) {
			fRec0[i4] = 0.f;
			
		}
		for (int i5 = 0; (i5 < 2); i5 = (i5 + 1)) {
			iRec4[i5] = 0;
			
		}
		fEntry6 = FAUSTFLOAT(3.693);
		for (int i6 = 0; (i6 < 2); i6 = (i6 + 1)) {
			fRec5[i6] = 0.f;
			
		}
		fEntry7 = FAUSTFLOAT(0.083);
		fEntry8 = FAUSTFLOAT(0.002);
		fEntry9 = FAUSTFLOAT(3.007);
		for (int i7 = 0; (i7 < 3); i7 = (i7 + 1)) {
			fRec6[i7] = 0.f;
			
		}
		for (int i8 = 0; (i8 < 2); i8 = (i8 + 1)) {
			iRec7[i8] = 0;
			
		}
		fEntry10 = FAUSTFLOAT(2.248);
		for (int i9 = 0; (i9 < 2); i9 = (i9 + 1)) {
			fRec8[i9] = 0.f;
			
		}
		fEntry11 = FAUSTFLOAT(0.087);
		fEntry12 = FAUSTFLOAT(0.002);
		fEntry13 = FAUSTFLOAT(4.968);
		for (int i10 = 0; (i10 < 3); i10 = (i10 + 1)) {
			fRec9[i10] = 0.f;
			
		}
		for (int i11 = 0; (i11 < 2); i11 = (i11 + 1)) {
			iRec10[i11] = 0;
			
		}
		fEntry14 = FAUSTFLOAT(2.828);
		for (int i12 = 0; (i12 < 2); i12 = (i12 + 1)) {
			fRec11[i12] = 0.f;
			
		}
		fEntry15 = FAUSTFLOAT(0.053);
		fEntry16 = FAUSTFLOAT(0.002);
		fEntry17 = FAUSTFLOAT(8.994);
		for (int i13 = 0; (i13 < 3); i13 = (i13 + 1)) {
			fRec12[i13] = 0.f;
			
		}
		for (int i14 = 0; (i14 < 2); i14 = (i14 + 1)) {
			iRec13[i14] = 0;
			
		}
		fEntry18 = FAUSTFLOAT(3.364);
		for (int i15 = 0; (i15 < 2); i15 = (i15 + 1)) {
			fRec14[i15] = 0.f;
			
		}
		fEntry19 = FAUSTFLOAT(0.053);
		fEntry20 = FAUSTFLOAT(0.002);
		fEntry21 = FAUSTFLOAT(12.006);
		for (int i16 = 0; (i16 < 3); i16 = (i16 + 1)) {
			fRec15[i16] = 0.f;
			
		}
		for (int i17 = 0; (i17 < 2); i17 = (i17 + 1)) {
			iRec16[i17] = 0;
			
		}
		fEntry22 = FAUSTFLOAT(2.488);
		for (int i18 = 0; (i18 < 2); i18 = (i18 + 1)) {
			fRec17[i18] = 0.f;
			
		}
		
	}
	
	virtual void init(int samplingFreq) {
		classInit(samplingFreq);
		instanceInit(samplingFreq);
	}
	
	virtual void buildUserInterface(UI* interface) {
		interface->openVerticalBox("0x00");
		interface->openHorizontalBox("1-excitator");
		interface->addNumEntry("decay", &fEntry5, 0.01f, 0.f, 1.f, 0.001f);
		interface->closeBox();
		interface->openHorizontalBox("2-resonator-0");
		interface->addNumEntry("amplitude0", &fEntry1, 0.167f, 0.f, 1.f, 0.001f);
		interface->addNumEntry("decay0", &fEntry6, 3.693f, 0.f, 10.f, 0.001f);
		interface->addNumEntry("harmonic0", &fEntry3, 1.f, 0.f, 50.f, 0.001f);
		interface->addNumEntry("rq0", &fEntry2, 0.002f, 0.f, 1.f, 0.0001f);
		interface->closeBox();
		interface->openHorizontalBox("2-resonator-1");
		interface->addNumEntry("amplitude1", &fEntry7, 0.083f, 0.f, 1.f, 0.001f);
		interface->addNumEntry("decay1", &fEntry10, 2.248f, 0.f, 10.f, 0.001f);
		interface->addNumEntry("harmonic1", &fEntry9, 3.007f, 0.f, 50.f, 0.001f);
		interface->addNumEntry("rq1", &fEntry8, 0.002f, 0.f, 1.f, 0.0001f);
		interface->closeBox();
		interface->openHorizontalBox("2-resonator-2");
		interface->addNumEntry("amplitude2", &fEntry11, 0.087f, 0.f, 1.f, 0.001f);
		interface->addNumEntry("decay2", &fEntry14, 2.828f, 0.f, 10.f, 0.001f);
		interface->addNumEntry("harmonic2", &fEntry13, 4.968f, 0.f, 50.f, 0.001f);
		interface->addNumEntry("rq2", &fEntry12, 0.002f, 0.f, 1.f, 0.0001f);
		interface->closeBox();
		interface->openHorizontalBox("2-resonator-3");
		interface->addNumEntry("amplitude3", &fEntry15, 0.053f, 0.f, 1.f, 0.001f);
		interface->addNumEntry("decay3", &fEntry18, 3.364f, 0.f, 10.f, 0.001f);
		interface->addNumEntry("harmonic3", &fEntry17, 8.994f, 0.f, 50.f, 0.001f);
		interface->addNumEntry("rq3", &fEntry16, 0.002f, 0.f, 1.f, 0.0001f);
		interface->closeBox();
		interface->openHorizontalBox("2-resonator-4");
		interface->addNumEntry("amplitude4", &fEntry19, 0.053f, 0.f, 1.f, 0.001f);
		interface->addNumEntry("decay4", &fEntry22, 2.488f, 0.f, 10.f, 0.001f);
		interface->addNumEntry("harmonic4", &fEntry21, 12.006f, 0.f, 50.f, 0.001f);
		interface->addNumEntry("rq4", &fEntry20, 0.002f, 0.f, 1.f, 0.0001f);
		interface->closeBox();
		interface->openVerticalBox("3-master");
		interface->addHorizontalSlider("pan", &fHslider1, 0.5f, 0.f, 1.f, 0.01f);
		interface->addHorizontalSlider("vol", &fHslider0, 0.5f, 0.f, 10.f, 0.01f);
		interface->closeBox();
		interface->addNumEntry("freq", &fEntry0, 440.f, 20.f, 20000.f, 1.f);
		interface->addNumEntry("gain", &fEntry4, 1.f, 0.f, 10.f, 0.01f);
		interface->addButton("gate",&fButton0);
		interface->closeBox();
		
	}
	
	virtual void compute(int count, FAUSTFLOAT** inputs, FAUSTFLOAT** outputs) {
		FAUSTFLOAT* output0 = outputs[0];
		FAUSTFLOAT* output1 = outputs[1];
		float fSlow0 = float(fHslider0);
		float fSlow1 = float(fHslider1);
		float fSlow2 = float(fEntry0);
		float fSlow3 = float((fConst1 * float((float((fSlow0 * float((1.f - fSlow1)))) / fSlow2))));
		float fSlow4 = float(fEntry2);
		float fSlow5 = float((float(fEntry1) / fSlow4));
		float fSlow6 = faustpower2_f(float((1.f - float((fConst2 * float((fSlow2 * fSlow4)))))));
		float fSlow7 = cosf(float((fConst3 * float((fSlow2 * float(fEntry3))))));
		float fSlow8 = float((1.f + fSlow6));
		float fSlow9 = float((4.f * float((fSlow7 / fSlow8))));
		float fSlow10 = float((float((1.f - fSlow6)) * sqrtf(float((1.f - float((4.f * float((float((fSlow6 * faustpower2_f(fSlow7))) / faustpower2_f(fSlow8))))))))));
		float fSlow11 = float((4.65661e-10f * float(fEntry4)));
		float fSlow12 = float(fButton0);
		int iSlow13 = int((fSlow12 > 0.f));
		float fSlow14 = float(fEntry5);
		float fSlow15 = float((1.f - powf(0.001f, float((1.f / float((float(int((fSlow14 == 0.f))) + float((float(iConst0) * fSlow14)))))))));
		int iSlow16 = int((fSlow12 <= 0.f));
		float fSlow17 = float(fEntry6);
		float fSlow18 = float((1.f - powf(0.001f, float((1.f / float((float(int((fSlow17 == 0.f))) + float((float(iConst0) * fSlow17)))))))));
		float fSlow19 = float(fEntry8);
		float fSlow20 = float((float(fEntry7) / fSlow19));
		float fSlow21 = faustpower2_f(float((1.f - float((fConst2 * float((fSlow2 * fSlow19)))))));
		float fSlow22 = cosf(float((fConst3 * float((fSlow2 * float(fEntry9))))));
		float fSlow23 = float((1.f + fSlow21));
		float fSlow24 = float((4.f * float((fSlow22 / fSlow23))));
		float fSlow25 = float((float((1.f - fSlow21)) * sqrtf(float((1.f - float((4.f * float((float((fSlow21 * faustpower2_f(fSlow22))) / faustpower2_f(fSlow23))))))))));
		float fSlow26 = float(fEntry10);
		float fSlow27 = float((1.f - powf(0.001f, float((1.f / float((float(int((fSlow26 == 0.f))) + float((float(iConst0) * fSlow26)))))))));
		float fSlow28 = float(fEntry12);
		float fSlow29 = float((float(fEntry11) / fSlow28));
		float fSlow30 = faustpower2_f(float((1.f - float((fConst2 * float((fSlow2 * fSlow28)))))));
		float fSlow31 = cosf(float((fConst3 * float((fSlow2 * float(fEntry13))))));
		float fSlow32 = float((1.f + fSlow30));
		float fSlow33 = float((4.f * float((fSlow31 / fSlow32))));
		float fSlow34 = float((float((1.f - fSlow30)) * sqrtf(float((1.f - float((4.f * float((float((fSlow30 * faustpower2_f(fSlow31))) / faustpower2_f(fSlow32))))))))));
		float fSlow35 = float(fEntry14);
		float fSlow36 = float((1.f - powf(0.001f, float((1.f / float((float(int((fSlow35 == 0.f))) + float((float(iConst0) * fSlow35)))))))));
		float fSlow37 = float(fEntry16);
		float fSlow38 = float((float(fEntry15) / fSlow37));
		float fSlow39 = faustpower2_f(float((1.f - float((fConst2 * float((fSlow2 * fSlow37)))))));
		float fSlow40 = cosf(float((fConst3 * float((fSlow2 * float(fEntry17))))));
		float fSlow41 = float((1.f + fSlow39));
		float fSlow42 = float((4.f * float((fSlow40 / fSlow41))));
		float fSlow43 = float((float((1.f - fSlow39)) * sqrtf(float((1.f - float((4.f * float((float((fSlow39 * faustpower2_f(fSlow40))) / faustpower2_f(fSlow41))))))))));
		float fSlow44 = float(fEntry18);
		float fSlow45 = float((1.f - powf(0.001f, float((1.f / float((float(int((fSlow44 == 0.f))) + float((float(iConst0) * fSlow44)))))))));
		float fSlow46 = float(fEntry20);
		float fSlow47 = float((float(fEntry19) / fSlow46));
		float fSlow48 = faustpower2_f(float((1.f - float((fConst2 * float((fSlow2 * fSlow46)))))));
		float fSlow49 = cosf(float((fConst3 * float((fSlow2 * float(fEntry21))))));
		float fSlow50 = float((1.f + fSlow48));
		float fSlow51 = float((4.f * float((fSlow49 / fSlow50))));
		float fSlow52 = float((float((1.f - fSlow48)) * sqrtf(float((1.f - float((4.f * float((float((faustpower2_f(fSlow49) * fSlow48)) / faustpower2_f(fSlow50))))))))));
		float fSlow53 = float(fEntry22);
		float fSlow54 = float((1.f - powf(0.001f, float((1.f / float((float(int((fSlow53 == 0.f))) + float((float(iConst0) * fSlow53)))))))));
		float fSlow55 = float((fConst1 * float((float((fSlow0 * fSlow1)) / fSlow2))));
		for (int i = 0; (i < count); i = (i + 1)) {
			iRec1[0] = int((iSlow13 & int((iRec1[1] | int((fRec2[1] >= 1.f))))));
			int iTemp0 = int((fRec2[1] > 0.f));
			fRec2[0] = float((float((float(int((int((int((iRec1[1] == 0)) & iSlow13)) & int((fRec2[1] < 1.f))))) + float((fRec2[1] * float((1.f - float((fSlow15 * float(int((iRec1[1] & iTemp0))))))))))) * float(int((int((int((iSlow16 & iTemp0)) == 0)) | int((fRec2[1] >= 1e-06f)))))));
			iRec3[0] = int((12345 + int((1103515245 * iRec3[1]))));
			float fTemp1 = float((fSlow11 * float((fRec2[0] * float(iRec3[0])))));
			fVec0[0] = fTemp1;
			float fTemp2 = float((fTemp1 - fVec0[1]));
			fRec0[0] = float((float((fSlow6 * float((float((fSlow9 * fRec0[1])) - fRec0[2])))) + float((fSlow10 * fTemp2))));
			iRec4[0] = int((iSlow13 & int((iRec4[1] | int((fRec5[1] >= 1.f))))));
			int iTemp3 = int((fRec5[1] > 0.f));
			fRec5[0] = float((float((float(int((int((int((iRec4[1] == 0)) & iSlow13)) & int((fRec5[1] < 1.f))))) + float((fRec5[1] * float((1.f - float((fSlow18 * float(int((iRec4[1] & iTemp3))))))))))) * float(int((int((int((iSlow16 & iTemp3)) == 0)) | int((fRec5[1] >= 1e-06f)))))));
			fRec6[0] = float((float((fSlow21 * float((float((fSlow24 * fRec6[1])) - fRec6[2])))) + float((fSlow25 * fTemp2))));
			iRec7[0] = int((iSlow13 & int((iRec7[1] | int((fRec8[1] >= 1.f))))));
			int iTemp4 = int((fRec8[1] > 0.f));
			fRec8[0] = float((float((float(int((int((int((iRec7[1] == 0)) & iSlow13)) & int((fRec8[1] < 1.f))))) + float((fRec8[1] * float((1.f - float((fSlow27 * float(int((iRec7[1] & iTemp4))))))))))) * float(int((int((int((iSlow16 & iTemp4)) == 0)) | int((fRec8[1] >= 1e-06f)))))));
			fRec9[0] = float((float((fSlow30 * float((float((fSlow33 * fRec9[1])) - fRec9[2])))) + float((fSlow34 * fTemp2))));
			iRec10[0] = int((iSlow13 & int((iRec10[1] | int((fRec11[1] >= 1.f))))));
			int iTemp5 = int((fRec11[1] > 0.f));
			fRec11[0] = float((float((float(int((int((int((iRec10[1] == 0)) & iSlow13)) & int((fRec11[1] < 1.f))))) + float((fRec11[1] * float((1.f - float((fSlow36 * float(int((iRec10[1] & iTemp5))))))))))) * float(int((int((int((iSlow16 & iTemp5)) == 0)) | int((fRec11[1] >= 1e-06f)))))));
			fRec12[0] = float((float((fSlow39 * float((float((fSlow42 * fRec12[1])) - fRec12[2])))) + float((fSlow43 * fTemp2))));
			iRec13[0] = int((iSlow13 & int((iRec13[1] | int((fRec14[1] >= 1.f))))));
			int iTemp6 = int((fRec14[1] > 0.f));
			fRec14[0] = float((float((float(int((int((int((iRec13[1] == 0)) & iSlow13)) & int((fRec14[1] < 1.f))))) + float((fRec14[1] * float((1.f - float((fSlow45 * float(int((iRec13[1] & iTemp6))))))))))) * float(int((int((int((iSlow16 & iTemp6)) == 0)) | int((fRec14[1] >= 1e-06f)))))));
			fRec15[0] = float((float((fSlow48 * float((float((fSlow51 * fRec15[1])) - fRec15[2])))) + float((fSlow52 * fTemp2))));
			iRec16[0] = int((iSlow13 & int((iRec16[1] | int((fRec17[1] >= 1.f))))));
			int iTemp7 = int((fRec17[1] > 0.f));
			fRec17[0] = float((float((float(int((int((int((iRec16[1] == 0)) & iSlow13)) & int((fRec17[1] < 1.f))))) + float((fRec17[1] * float((1.f - float((fSlow54 * float(int((iRec16[1] & iTemp7))))))))))) * float(int((int((int((iSlow16 & iTemp7)) == 0)) | int((fRec17[1] >= 1e-06f)))))));
			float fTemp8 = float((float((float((float((float((fSlow5 * float((fRec0[0] * fRec5[0])))) + float((fSlow20 * float((fRec6[0] * fRec8[0])))))) + float((fSlow29 * float((fRec9[0] * fRec11[0])))))) + float((fSlow38 * float((fRec12[0] * fRec14[0])))))) + float((fSlow47 * float((fRec15[0] * fRec17[0]))))));
			output0[i] = FAUSTFLOAT(float((fSlow3 * fTemp8)));
			output1[i] = FAUSTFLOAT(float((fSlow55 * fTemp8)));
			iRec1[1] = iRec1[0];
			fRec2[1] = fRec2[0];
			iRec3[1] = iRec3[0];
			fVec0[1] = fVec0[0];
			fRec0[2] = fRec0[1];
			fRec0[1] = fRec0[0];
			iRec4[1] = iRec4[0];
			fRec5[1] = fRec5[0];
			fRec6[2] = fRec6[1];
			fRec6[1] = fRec6[0];
			iRec7[1] = iRec7[0];
			fRec8[1] = fRec8[0];
			fRec9[2] = fRec9[1];
			fRec9[1] = fRec9[0];
			iRec10[1] = iRec10[0];
			fRec11[1] = fRec11[0];
			fRec12[2] = fRec12[1];
			fRec12[1] = fRec12[0];
			iRec13[1] = iRec13[0];
			fRec14[1] = fRec14[0];
			fRec15[2] = fRec15[1];
			fRec15[1] = fRec15[0];
			iRec16[1] = iRec16[0];
			fRec17[1] = fRec17[0];
			
		}
		
	}

	
};


#include <stdio.h>
#include <string>
#include "m_pd.h"

#define sym(name) xsym(name)
#define xsym(name) #name
#define faust_setup(name) xfaust_setup(name)
#define xfaust_setup(name) name ## _tilde_setup(void)

// time for "active" toggle xfades in secs
#define XFADE_TIME 0.1f

static t_class *faust_class;

struct t_faust {
  t_object x_obj;
#ifdef __MINGW32__
  /* This seems to be necessary as some as yet undetermined Pd routine seems
     to write past the end of x_obj on Windows. */
  int fence; /* dummy field (not used) */
#endif
  mydsp *dsp;
  PdUI *ui;
  std::string *label;
  int active, xfade, n_xfade, rate, n_in, n_out;
  t_sample **inputs, **outputs, **buf;
  t_outlet *out;
  t_sample f;
};

static t_symbol *s_button, *s_checkbox, *s_vslider, *s_hslider, *s_nentry,
  *s_vbargraph, *s_hbargraph;

static inline void zero_samples(int k, int n, t_sample **out)
{
  for (int i = 0; i < k; i++)
#ifdef __STDC_IEC_559__
    /* IEC 559 a.k.a. IEEE 754 floats can be initialized faster like this */
    memset(out[i], 0, n*sizeof(t_sample));
#else
    for (int j = 0; j < n; j++)
      out[i][j] = 0.0f;
#endif
}

static inline void copy_samples(int k, int n, t_sample **out, t_sample **in)
{
  for (int i = 0; i < k; i++)
    memcpy(out[i], in[i], n*sizeof(t_sample));
}

static t_int *faust_perform(t_int *w)
{
  t_faust *x = (t_faust *)(w[1]);
  int n = (int)(w[2]);
  if (!x->dsp || !x->buf) return (w+3);
  AVOIDDENORMALS;
  if (x->xfade > 0) {
    float d = 1.0f/x->n_xfade, f = (x->xfade--)*d;
    d = d/n;
    x->dsp->compute(n, x->inputs, x->buf);
    if (x->active)
      if (x->n_in == x->n_out)
	/* xfade inputs -> buf */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->inputs[i][j]+(1.0f-f)*x->buf[i][j];
      else
	/* xfade 0 -> buf */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = (1.0f-f)*x->buf[i][j];
    else
      if (x->n_in == x->n_out)
	/* xfade buf -> inputs */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->buf[i][j]+(1.0f-f)*x->inputs[i][j];
      else
	/* xfade buf -> 0 */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->buf[i][j];
  } else if (x->active) {
    x->dsp->compute(n, x->inputs, x->buf);
    copy_samples(x->n_out, n, x->outputs, x->buf);
  } else if (x->n_in == x->n_out) {
    copy_samples(x->n_out, n, x->buf, x->inputs);
    copy_samples(x->n_out, n, x->outputs, x->buf);
  } else
    zero_samples(x->n_out, n, x->outputs);
  return (w+3);
}

static void faust_dsp(t_faust *x, t_signal **sp)
{
  int n = sp[0]->s_n, sr = (int)sp[0]->s_sr;
  if (x->rate <= 0) {
    /* default sample rate is whatever Pd tells us */
    PdUI *ui = x->ui;
    float *z = NULL;
    if (ui->nelems > 0 &&
	(z = (float*)malloc(ui->nelems*sizeof(float)))) {
      /* save the current control values */
      for (int i = 0; i < ui->nelems; i++)
	if (ui->elems[i].zone)
	  z[i] = *ui->elems[i].zone;
    }
    /* set the proper sample rate; this requires reinitializing the dsp */
    x->rate = sr;
    x->dsp->init(sr);
    if (z) {
      /* restore previous control values */
      for (int i = 0; i < ui->nelems; i++)
	if (ui->elems[i].zone)
	  *ui->elems[i].zone = z[i];
      free(z);
    }
  }
  if (n > 0)
    x->n_xfade = (int)(x->rate*XFADE_TIME/n);
  dsp_add(faust_perform, 2, x, n);
  for (int i = 0; i < x->n_in; i++)
    x->inputs[i] = sp[i+1]->s_vec;
  for (int i = 0; i < x->n_out; i++)
    x->outputs[i] = sp[x->n_in+i+1]->s_vec;
  if (x->buf != NULL)
    for (int i = 0; i < x->n_out; i++) {
      x->buf[i] = (t_sample*)malloc(n*sizeof(t_sample));
      if (x->buf[i] == NULL) {
	for (int j = 0; j < i; j++)
	  free(x->buf[j]);
	free(x->buf);
	x->buf = NULL;
	break;
      }
    }
}

static int pathcmp(const char *s, const char *t)
{
  int n = strlen(s), m = strlen(t);
  if (n == 0 || m == 0)
    return 0;
  else if (t[0] == '/')
    return strcmp(s, t);
  else if (n <= m || s[n-m-1] != '/')
    return strcmp(s+1, t);
  else
    return strcmp(s+n-m, t);
}

static void faust_any(t_faust *x, t_symbol *s, int argc, t_atom *argv)
{
  if (!x->dsp) return;
  PdUI *ui = x->ui;
  if (s == &s_bang) {
    for (int i = 0; i < ui->nelems; i++)
      if (ui->elems[i].label && ui->elems[i].zone) {
	t_atom args[6];
	t_symbol *_s;
	switch (ui->elems[i].type) {
	case UI_BUTTON:
	  _s = s_button;
	  break;
	case UI_CHECK_BUTTON:
	  _s = s_checkbox;
	  break;
	case UI_V_SLIDER:
	  _s = s_vslider;
	  break;
	case UI_H_SLIDER:
	  _s = s_hslider;
	  break;
	case UI_NUM_ENTRY:
	  _s = s_nentry;
	  break;
	case UI_V_BARGRAPH:
	  _s = s_vbargraph;
	  break;
	case UI_H_BARGRAPH:
	  _s = s_hbargraph;
	  break;
	default:
	  continue;
	}
	SETSYMBOL(&args[0], gensym(ui->elems[i].label));
	SETFLOAT(&args[1], *ui->elems[i].zone);
	SETFLOAT(&args[2], ui->elems[i].init);
	SETFLOAT(&args[3], ui->elems[i].min);
	SETFLOAT(&args[4], ui->elems[i].max);
	SETFLOAT(&args[5], ui->elems[i].step);
	outlet_anything(x->out, _s, 6, args);
      }
  } else {
    const char *label = s->s_name;
    int count = 0;
    for (int i = 0; i < ui->nelems; i++)
      if (ui->elems[i].label &&
	  pathcmp(ui->elems[i].label, label) == 0) {
	if (argc == 0) {
	  if (ui->elems[i].zone) {
	    t_atom arg;
	    SETFLOAT(&arg, *ui->elems[i].zone);
	    outlet_anything(x->out, gensym(ui->elems[i].label), 1, &arg);
	  }
	  ++count;
	} else if (argc == 1 &&
		   (argv[0].a_type == A_FLOAT ||
		    argv[0].a_type == A_DEFFLOAT) &&
		   ui->elems[i].zone) {
	  float f = atom_getfloat(argv);
	  *ui->elems[i].zone = f;
	  ++count;
	} else
	  pd_error(x, "[faust] %s: bad control argument: %s",
		   x->label->c_str(), label);
      }
    if (count == 0 && strcmp(label, "active") == 0) {
      if (argc == 0) {
	t_atom arg;
	SETFLOAT(&arg, (float)x->active);
	outlet_anything(x->out, gensym((char*)"active"), 1, &arg);
      } else if (argc == 1 &&
		 (argv[0].a_type == A_FLOAT ||
		  argv[0].a_type == A_DEFFLOAT)) {
	float f = atom_getfloat(argv);
	x->active = (int)f;
	x->xfade = x->n_xfade;
      }
    }
  }
}

static void faust_free(t_faust *x)
{
  if (x->label) delete x->label;
  if (x->dsp) delete x->dsp;
  if (x->ui) delete x->ui;
  if (x->inputs) free(x->inputs);
  if (x->outputs) free(x->outputs);
  if (x->buf) {
    for (int i = 0; i < x->n_out; i++)
      if (x->buf[i]) free(x->buf[i]);
    free(x->buf);
  }
}

static void *faust_new(t_symbol *s, int argc, t_atom *argv)
{
  t_faust *x = (t_faust*)pd_new(faust_class);
  int sr = -1;
  t_symbol *id = NULL;
  x->active = 1;
  for (int i = 0; i < argc; i++)
    if (argv[i].a_type == A_FLOAT || argv[i].a_type == A_DEFFLOAT)
      sr = (int)argv[i].a_w.w_float;
    else if (argv[i].a_type == A_SYMBOL || argv[i].a_type == A_DEFSYMBOL)
      id = argv[i].a_w.w_symbol;
  x->rate = sr;
  if (sr <= 0) sr = 44100;
  x->xfade = 0; x->n_xfade = (int)(sr*XFADE_TIME/64);
  x->inputs = x->outputs = x->buf = NULL;
    x->label = new std::string(sym(mydsp) "~");
  x->dsp = new mydsp();
  x->ui = new PdUI(sym(mydsp), id?id->s_name:NULL);
  if (!x->dsp || !x->ui || !x->label) goto error;
  if (id) {
    *x->label += " ";
    *x->label += id->s_name;
  }
  x->n_in = x->dsp->getNumInputs();
  x->n_out = x->dsp->getNumOutputs();
  if (x->n_in > 0)
    x->inputs = (t_sample**)malloc(x->n_in*sizeof(t_sample*));
  if (x->n_out > 0) {
    x->outputs = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
    x->buf = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
  }
  if ((x->n_in > 0 && x->inputs == NULL) ||
      (x->n_out > 0 && (x->outputs == NULL || x->buf == NULL)))
    goto error;
  for (int i = 0; i < x->n_out; i++)
    x->buf[i] = NULL;
  x->dsp->init(sr);
  x->dsp->buildUserInterface(x->ui);
  for (int i = 0; i < x->n_in; i++)
    inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
  x->out = outlet_new(&x->x_obj, 0);
  for (int i = 0; i < x->n_out; i++)
    outlet_new(&x->x_obj, &s_signal);
  return (void *)x;
 error:
  faust_free(x);
  x->dsp = NULL; x->ui = NULL;
  x->inputs = x->outputs = x->buf = NULL;
  return (void *)x;
}

extern "C" void faust_setup(mydsp)
{
  t_symbol *s = gensym(sym(mydsp) "~");
  faust_class =
    class_new(s, (t_newmethod)faust_new, (t_method)faust_free,
	      sizeof(t_faust), CLASS_DEFAULT,
	      A_GIMME, A_NULL);
  class_addmethod(faust_class, (t_method)faust_dsp, gensym((char*)"dsp"), A_NULL);
  class_addanything(faust_class, faust_any);
  class_addmethod(faust_class, nullfn, &s_signal, A_NULL);
  s_button = gensym((char*)"button");
  s_checkbox = gensym((char*)"checkbox");
  s_vslider = gensym((char*)"vslider");
  s_hslider = gensym((char*)"hslider");
  s_nentry = gensym((char*)"nentry");
  s_vbargraph = gensym((char*)"vbargraph");
  s_hbargraph = gensym((char*)"hbargraph");
  /* give some indication that we're loaded and ready to go */
  mydsp dsp = mydsp();
  post("[faust] %s: %d inputs, %d outputs", sym(mydsp) "~",
       dsp.getNumInputs(), dsp.getNumOutputs());
}

#endif
